package com.elab.log.log4j;

import ch.qos.logback.classic.Level;
import ch.qos.logback.classic.spi.ILoggingEvent;
import ch.qos.logback.classic.spi.ThrowableProxy;
import ch.qos.logback.core.AppenderBase;
import com.dianping.cat.Cat;

import java.io.PrintWriter;
import java.io.StringWriter;

/**
 * CAT集成Logback拓展
 *
 * @author Liukx
 * @create 2018-02-06 17:58
 * @email liukx@elab-plus.com
 **/
public class CatLogbackLog extends AppenderBase<ILoggingEvent> {

    private final String type = "Logback";

    /**
     * 字符达到多少长度进行换行
     * <code>
     *     <appender name="cat" class="com.elab.log.log4j.CatLogbackLog">
     *         <newLineNumber>255</newLineNumber>
     *         <maxCharNumber>2888</maxCharNumber>
     *     </appender>
     * </code>
     *
     */
    private int newLineNumber = 200;
    /**
     * 字符达到多少长度则告警
     */
    private int maxCharNumber = 2000;

    /**
     * 字符超过最大长度
     */
    private String charOverflowName = "charOverflow";

    public void setCharOverflowName(String charOverflowName) {
        this.charOverflowName = charOverflowName;
    }

    public void setNewLineNumber(int newLineNumber) {
        this.newLineNumber = newLineNumber;
    }

    public void setMaxCharNumber(int maxCharNumber) {
        this.maxCharNumber = maxCharNumber;
    }

    @Override
    protected void append(ILoggingEvent event) {
        // 判断配置
        boolean isTraceMode = Cat.getManager().isTraceMode();

        Level level = event.getLevel();

        if (level.isGreaterOrEqual(Level.ERROR)) {
            this.logError(event);
        } else if (!isTraceMode) {
            this.logTrace(event);
        }
    }

    /**
     * 处理异常栈
     *
     * @param exception
     * @return
     */
    private String buildExceptionStack(Throwable exception) {
        if (exception != null) {
            StringWriter writer = new StringWriter(2048);
            exception.printStackTrace(new PrintWriter(writer));
            return writer.toString();
        } else {
            return "";
        }
    }

    /**
     * 打印错误日志
     *
     * @param event
     */
    private void logError(ILoggingEvent event) {
        ThrowableProxy info = (ThrowableProxy)event.getThrowableProxy();
        if (info != null) {
            Throwable exception = info.getThrowable();
            Object message = event.getMessage();
            if (message != null) {
                Cat.logError(String.valueOf(message), exception);
            } else {
                Cat.logError(exception);
            }
        }

    }

    private void logTrace(ILoggingEvent event) {
        String name = event.getLevel().toString();
        String message = event.getMessage();
        int length = message.length();
        if (length > maxCharNumber) {
            // 将范围控制在20个区间范围内
            int eventValue = Math.floorMod(length, 20);
            Cat.logEvent(charOverflowName, eventValue + "");
        }

        Cat.logEvent(type, name, "0", formatShowData(message));
    }

    /**
     * 根据打印出来的内容进行格式化,按照长度进行换行
     *
     * @param msg
     * @return
     */
    public String formatShowData(String msg) {
        int length = msg.length();

        int lineLength = newLineNumber;

        if (length < lineLength) {
            return msg;
        }

        int maxSplit = length / lineLength;
        int mo = length % lineLength;

        if (length % lineLength > 0) {
            maxSplit++;
        }

        StringBuilder sb = new StringBuilder();

        for (int i = 0; i < maxSplit; i++) {
            int index = i * lineLength;
            if (maxSplit == (i + 1)) {
                sb.append(msg, index, index + mo).append("\n");
            } else {
                sb.append(msg, index, index + lineLength).append("\n");
            }
        }
        return sb.toString();
    }

}
